@@ -2,4 +2,6 @@
 # the trustworthiness of interpreter algorithms.
 
 from .abc_evaluator import InterpreterEvaluator
-from .deletion_insertion import DeletionInsertion
\ No newline at end of file
+from .deletion_insertion import DeletionInsertion
+from .perturbation import Perturbation
+from .localization import PointGame, PointGameSegmentation
\ No newline at end of file
